package pk

import "net/http"

type Preparer interface {
	Prepare(*http.Request) (*http.Request, error)
}

type PreparerFunc func(*http.Request) (*http.Request, error)

func (pf PreparerFunc) Prepare(r *http.Request) (*http.Request, error) {
	return pf(r)
}

type PrepareDecorator func(Preparer) Preparer

func AsPost() PrepareDecorator { return WithMethod("POST") }

func WithMethod(method string) PrepareDecorator {
	return func(p Preparer) Preparer {
		return PreparerFunc(func(r *http.Request) (*http.Request, error) {
			r.Method = method
			return p.Prepare(r)
		})
	}
}
